/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "../client/client.h"
#include "../client/qmenu.h"

#define REF_SOFT	0
#define REF_OPENGL	1
#define REF_3DFX	2
#define REF_POWERVR	3
#define REF_VERITE	4
#define FONTSIZE 14

extern cvar_t *vid_ref;
extern cvar_t *vid_fullscreen;
extern cvar_t *vid_gamma;
extern cvar_t *scr_viewsize;

static cvar_t  *gl_waterwarp;
static cvar_t  *gl_waterwaves;

static cvar_t *gl_mode;
static cvar_t *gl_driver;
static cvar_t *gl_picmip;

static cvar_t *gl_shadows;
static cvar_t *gl_ext_multitexture;

static cvar_t *gl_ext_palettedtexture;
static cvar_t *gl_finish;

static cvar_t *sw_mode;
static cvar_t *sw_stipplealpha;

static cvar_t *gl_texturemode;

extern void M_ForceMenuOff( void );

//#define CHARWIDTH  (viddef.width  * 0.0125)
#define CHARHEIGHT (viddef.height * 0.035)


/*
====================================================================

MENU INTERACTION

====================================================================
*/
#define SOFTWARE_MENU 0
#define OPENGL_MENU   1

static menuframework_s  s_software_menu;
static menuframework_s	s_opengl_menu;
static menuframework_s *s_current_menu;
static int				s_current_menu_index;

static menulist_s		s_mode_list[2];
static menulist_s		s_ref_list[2];
static menuslider_s		s_tq_slider;
static menuslider_s		s_screensize_slider[2];
static menuslider_s		s_brightness_slider[2];
static menulist_s  		s_fs_box[2];
static menulist_s  		s_stipple_box;
static menulist_s  		s_paletted_texture_box;
static menulist_s  		s_finish_box;
static menuaction_s		s_cancel_action[2];
static menuaction_s		s_defaults_action[2];
static menulist_s               s_tfilter_box;
static  menulist_s              s_gl_waterwarp_box;
static  menulist_s              s_gl_waterwaves_box;


static void DriverCallback( void *unused )
{
	s_ref_list[!s_current_menu_index].curvalue = s_ref_list[s_current_menu_index].curvalue;

	if ( s_ref_list[s_current_menu_index].curvalue == 0 )
	{
		s_current_menu = &s_software_menu;
		s_current_menu_index = 0;
	}
	else
	{
		s_current_menu = &s_opengl_menu;
		s_current_menu_index = 1;
	}

}


static void glwaterwavescallback (void *s)
{
        Cvar_SetValue("gl_waterwaves",s_gl_waterwaves_box.curvalue);
}


static void glwaterwarpcallback (void *s)
{
        Cvar_SetValue("gl_waterwarp",s_gl_waterwarp_box.curvalue);
}

static void ScreenSizeCallback( void *s )
{
	menuslider_s *slider = ( menuslider_s * ) s;

	Cvar_SetValue( "viewsize", slider->curvalue * 10 );
}

static void BrightnessCallback( void *s )
{
	menuslider_s *slider = ( menuslider_s * ) s;

	if ( s_current_menu_index == SOFTWARE_MENU )
		s_brightness_slider[1].curvalue = s_brightness_slider[0].curvalue;
	else
		s_brightness_slider[0].curvalue = s_brightness_slider[1].curvalue;

	if ( stricmp( vid_ref->string, "soft" ) == 0 )
	{
		float gamma = ( 0.8 - ( slider->curvalue/10.0 - 0.5 ) ) + 0.5;

		Cvar_SetValue( "vid_gamma", gamma );
	}
}

static void ResetDefaults( void *unused )
{
	VID_MenuInit();
}

static void ApplyChanges( void *unused )
{
        //knightmare
        if (s_tfilter_box.curvalue == 0)
		Cvar_Set("gl_texturemode", "GL_NEAREST");
        else if (s_tfilter_box.curvalue == 1)
		Cvar_Set("gl_texturemode", "GL_NEAREST_MIPMAP_NEAREST");
        else if (s_tfilter_box.curvalue == 2)
		Cvar_Set("gl_texturemode", "GL_LINEAR");
        else if (s_tfilter_box.curvalue == 3)
		Cvar_Set("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST");
        else
		Cvar_Set("gl_texturemode", "GL_LINEAR_MIPMAP_LINEAR");



	float gamma;

	/*
	** make values consistent
	*/
	s_fs_box[!s_current_menu_index].curvalue = s_fs_box[s_current_menu_index].curvalue;
	s_brightness_slider[!s_current_menu_index].curvalue = s_brightness_slider[s_current_menu_index].curvalue;

        //s_ref_list[!s_current_menu_index].curvalue = s_ref_list[s_current_menu_index].curvalue;

	/*
	** invert sense so greater = brighter, and scale to a range of 0.5 to 1.3
	*/
	gamma = ( 0.8 - ( s_brightness_slider[s_current_menu_index].curvalue/10.0 - 0.5 ) ) + 0.5;



	Cvar_SetValue( "vid_gamma", gamma );
        //Cvar_SetValue( "sw_stipplealpha", s_stipple_box.curvalue );
        //Cvar_SetValue( "gl_picmip", 3 - s_tq_slider.curvalue );
	Cvar_SetValue( "vid_fullscreen", s_fs_box[s_current_menu_index].curvalue );

        Cvar_SetValue( "gl_waterwarp", s_gl_waterwarp_box.curvalue );
        Cvar_SetValue( "gl_waterwaves", s_gl_waterwaves_box.curvalue );



        Cvar_SetValue( "gl_shadows", s_finish_box.curvalue );
        Cvar_SetValue( "gl_ext_multitexture", s_paletted_texture_box.curvalue );

        //Cvar_SetValue( "gl_ext_palettedtexture", s_paletted_texture_box.curvalue );
        //Cvar_SetValue( "gl_finish", s_finish_box.curvalue );
        //Cvar_SetValue( "sw_mode", s_mode_list[SOFTWARE_MENU].curvalue );
	Cvar_SetValue( "gl_mode", s_mode_list[OPENGL_MENU].curvalue );


        /*
	switch ( s_ref_list[s_current_menu_index].curvalue )
	{
	case REF_SOFT:
		Cvar_Set( "vid_ref", "soft" );
		break;
	case REF_OPENGL:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "opengl32" );
		break;
	case REF_3DFX:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "3dfxgl" );
		break;
	case REF_POWERVR:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "pvrgl" );
		break;
	case REF_VERITE:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "veritegl" );
		break;
	}
        */

	/*
	** update appropriate stuff if we're running OpenGL and gamma
	** has been modified
	*/
	if ( stricmp( vid_ref->string, "gl" ) == 0 )
	{
		if ( vid_gamma->modified )
		{
			vid_ref->modified = true;
			if ( stricmp( gl_driver->string, "3dfxgl" ) == 0 )
			{
				char envbuffer[1024];
				float g;

				vid_ref->modified = true;

				g = 2.00 * ( 0.8 - ( vid_gamma->value - 0.5 ) ) + 1.0F;
				Com_sprintf( envbuffer, sizeof(envbuffer), "SSTV2_GAMMA=%f", g );
				putenv( envbuffer );
				Com_sprintf( envbuffer, sizeof(envbuffer), "SST_GAMMA=%f", g );
				putenv( envbuffer );

				vid_gamma->modified = false;
			}
		}

		if ( gl_driver->modified )
			vid_ref->modified = true;
	}



	M_ForceMenuOff();
}

static void CancelChanges( void *unused )
{
	extern void M_PopMenu( void );

	M_PopMenu();
}




// Knightmare added
int texfilter_box_setval (void)
{
	if (!Q_strcasecmp(gl_texturemode->string, "GL_NEAREST"))
		return 0;
	else if (!Q_strcasecmp(gl_texturemode->string, "GL_NEAREST_MIPMAP_NEAREST"))
		return 1;
	else if (!Q_strcasecmp(gl_texturemode->string, "GL_LINEAR"))
		return 2;
	else if (!Q_strcasecmp(gl_texturemode->string, "GL_LINEAR_MIPMAP_NEAREST"))
                return 3;
	else // if (!Q_strcasecmp(gl_texturemode->string, "GL_LINEAR_MIPMAP_LINEAR"))
                return 4;
}




/*
** VID_MenuInit
*/
void VID_MenuInit( void )
{
	static const char *resolutions[] = 
	{
                "320 x 240",
                "400 x 300",
                "512 x 384",
                "640 x 480",
                "800 x 600",
                "960 x 720",
                "1024 x 768",
                "1152 x 864",
                "1280 x 960",
                "1600 x 1200",
                "2048 x 1536",
                "1600 x 900",
		0
	};
	static const char *refs[] =
	{
                "[software ]",
                "[OpenGL   ]",
                //"[3Dfx OpenGL   ]",
                //"[PowerVR OpenGL]",
//              //"[Rendition OpenGL]",
		0
	};
	static const char *yesno_names[] =
	{
		"no",
		"yes",
		0
	};
        //filter control, from knightmare
	static const char *mip_names[] =
	{
		"none",
		"nearest",
		"linear",
		"bilinear",
		"trilinear",
		0
	};

	int i;

	if ( !gl_driver )
		gl_driver = Cvar_Get( "gl_driver", "opengl32", 0 );
	if ( !gl_picmip )
		gl_picmip = Cvar_Get( "gl_picmip", "0", 0 );
	if ( !gl_mode )
		gl_mode = Cvar_Get( "gl_mode", "3", 0 );
	if ( !sw_mode )
		sw_mode = Cvar_Get( "sw_mode", "0", 0 );
	if ( !gl_ext_palettedtexture )
                gl_ext_palettedtexture = Cvar_Get( "gl_ext_palettedtexture", "0", CVAR_ARCHIVE );
	if ( !gl_finish )
		gl_finish = Cvar_Get( "gl_finish", "0", CVAR_ARCHIVE );
	if ( !gl_texturemode )
		gl_texturemode = Cvar_Get( "gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE );
        if ( !gl_shadows )
                gl_shadows = Cvar_Get( "gl_shadows", "1", 0 );
        if ( !gl_ext_multitexture )
                gl_ext_multitexture = Cvar_Get( "gl_ext_multitexture", "1", 0 );
        if ( !gl_waterwarp )
                gl_waterwarp = Cvar_Get( "gl_waterwarp", "1", 0 );
        if ( !gl_waterwaves )
                gl_waterwaves = Cvar_Get( "gl_waterwaves", "1", 0 );




	if ( !sw_stipplealpha )
		sw_stipplealpha = Cvar_Get( "sw_stipplealpha", "0", CVAR_ARCHIVE );

	s_mode_list[SOFTWARE_MENU].curvalue = sw_mode->value;
	s_mode_list[OPENGL_MENU].curvalue = gl_mode->value;

	if ( !scr_viewsize )
		scr_viewsize = Cvar_Get ("viewsize", "100", CVAR_ARCHIVE);

	s_screensize_slider[SOFTWARE_MENU].curvalue = scr_viewsize->value/10;
	s_screensize_slider[OPENGL_MENU].curvalue = scr_viewsize->value/10;

        /*
	if ( strcmp( vid_ref->string, "soft" ) == 0 )
	{
		s_current_menu_index = SOFTWARE_MENU;
		s_ref_list[0].curvalue = s_ref_list[1].curvalue = REF_SOFT;
	}
	else if ( strcmp( vid_ref->string, "gl" ) == 0 )
	{
		s_current_menu_index = OPENGL_MENU;
		if ( strcmp( gl_driver->string, "3dfxgl" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_3DFX;
		else if ( strcmp( gl_driver->string, "pvrgl" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_POWERVR;
		else if ( strcmp( gl_driver->string, "opengl32" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_OPENGL;
		else
//			s_ref_list[s_current_menu_index].curvalue = REF_VERITE;
			s_ref_list[s_current_menu_index].curvalue = REF_OPENGL;
	}
        */



	{
		s_current_menu_index = OPENGL_MENU;
		if ( strcmp( gl_driver->string, "3dfxgl" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_3DFX;
		else if ( strcmp( gl_driver->string, "pvrgl" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_POWERVR;
		else if ( strcmp( gl_driver->string, "opengl32" ) == 0 )
			s_ref_list[s_current_menu_index].curvalue = REF_OPENGL;
		else
//			s_ref_list[s_current_menu_index].curvalue = REF_VERITE;
			s_ref_list[s_current_menu_index].curvalue = REF_OPENGL;
	}



	s_software_menu.x = viddef.width * 0.50;
	s_software_menu.nitems = 0;
	s_opengl_menu.x = viddef.width * 0.50;
	s_opengl_menu.nitems = 0;

	for ( i = 0; i < 2; i++ )
	{
                /*
                s_ref_list[i].generic.type = MTYPE_SPINCONTROL;
                s_ref_list[i].generic.name = "Video Driver";
		s_ref_list[i].generic.x = 0;
		s_ref_list[i].generic.y = 0;
		s_ref_list[i].generic.callback = DriverCallback;
		s_ref_list[i].itemnames = refs;
                */

		s_mode_list[i].generic.type = MTYPE_SPINCONTROL;
                s_mode_list[i].generic.name = "Resolution";
		s_mode_list[i].generic.x = 0;
                s_mode_list[i].generic.y = 1.25 * CHARHEIGHT;
                s_mode_list[i].generic.statusbar = "gl_mode";
		s_mode_list[i].itemnames = resolutions;


		s_fs_box[i].generic.type = MTYPE_SPINCONTROL;
		s_fs_box[i].generic.x	= 0;
                s_fs_box[i].generic.y   = 3.75 * CHARHEIGHT;
                s_fs_box[i].generic.name        = "Fullscreen";
		s_fs_box[i].itemnames = yesno_names;
                s_fs_box[i].generic.statusbar = "vid_fullscreen";
		s_fs_box[i].curvalue = vid_fullscreen->value;



		s_brightness_slider[i].generic.type	= MTYPE_SLIDER;
		s_brightness_slider[i].generic.x	= 0;
                s_brightness_slider[i].generic.y        = 6.25 * CHARHEIGHT;
                s_brightness_slider[i].generic.name     = "Brightness";
		s_brightness_slider[i].generic.callback = BrightnessCallback;
		s_brightness_slider[i].minvalue = 5;
		s_brightness_slider[i].maxvalue = 13;
                s_brightness_slider[i].generic.statusbar = "vid_gamma";
		s_brightness_slider[i].curvalue = ( 1.3 - vid_gamma->value + 0.5 ) * 10;




		s_screensize_slider[i].generic.type	= MTYPE_SLIDER;
		s_screensize_slider[i].generic.x		= 0;
                s_screensize_slider[i].generic.y                = 7.5 * CHARHEIGHT;
                s_screensize_slider[i].generic.name     = "Screen Size";
                s_screensize_slider[i].minvalue = 4;
                s_screensize_slider[i].maxvalue = 10;
                s_screensize_slider[i].generic.statusbar = "viewsize";
		s_screensize_slider[i].generic.callback = ScreenSizeCallback;


               /*
                s_defaults_action[i].generic.type = MTYPE_ACTION;
		s_defaults_action[i].generic.name = "reset to defaults";
		s_defaults_action[i].generic.x    = 0;
                s_defaults_action[i].generic.y    = 100;
		s_defaults_action[i].generic.callback = ResetDefaults;
                */

		s_cancel_action[i].generic.type = MTYPE_ACTION;
                s_cancel_action[i].generic.name = "Apply Settings";
		s_cancel_action[i].generic.x    = 0;
                s_cancel_action[i].generic.y    = 15.25 * CHARHEIGHT;
                s_cancel_action[i].generic.statusbar = "confirm settings";
                //s_cancel_action[i].generic.callback = CancelChanges;
                s_cancel_action[i].generic.callback = ApplyChanges;
	}

        s_tfilter_box.generic.type = MTYPE_SPINCONTROL;
        s_tfilter_box.generic.x = 0;
        s_tfilter_box.generic.y = 2.5 * CHARHEIGHT;
        s_tfilter_box.generic.name      = "Texture Filter";
        s_tfilter_box.generic.statusbar = "gl_texturemode";
        s_tfilter_box.itemnames = mip_names;
        s_tfilter_box.curvalue = texfilter_box_setval();


        /*
	s_stipple_box.generic.type = MTYPE_SPINCONTROL;
	s_stipple_box.generic.x	= 0;
	s_stipple_box.generic.y	= 60;
	s_stipple_box.generic.name	= "stipple alpha";
	s_stipple_box.curvalue = sw_stipplealpha->value;
	s_stipple_box.itemnames = yesno_names;
        */


        /*
	s_tq_slider.generic.type	= MTYPE_SLIDER;
	s_tq_slider.generic.x		= 0;
	s_tq_slider.generic.y		= 60;
	s_tq_slider.generic.name	= "texture quality";
	s_tq_slider.minvalue = 0;
	s_tq_slider.maxvalue = 3;
	s_tq_slider.curvalue = 3-gl_picmip->value;
        */


        //============cheap hack=============================
        // this is now the shadows
	s_finish_box.generic.type = MTYPE_SPINCONTROL;
	s_finish_box.generic.x	= 0;
        s_finish_box.generic.y  = 10 * CHARHEIGHT;
        s_finish_box.generic.name       = "Shadows";
        s_finish_box.generic.statusbar = "gl_shadows";
        s_finish_box.curvalue = gl_shadows->value;
	s_finish_box.itemnames = yesno_names;

        //================ cheap hack =============================
        // this is for detailtextures

	s_paletted_texture_box.generic.type = MTYPE_SPINCONTROL;
	s_paletted_texture_box.generic.x	= 0;
        s_paletted_texture_box.generic.y        = 11.25 * CHARHEIGHT;
        s_paletted_texture_box.generic.name     = "Detail Textures";
        s_paletted_texture_box.generic.statusbar = "gl_ext_multitexture";
        s_paletted_texture_box.curvalue = gl_ext_multitexture->value;
        s_paletted_texture_box.itemnames = yesno_names;

        s_gl_waterwarp_box.generic.type =       MTYPE_SPINCONTROL;
        s_gl_waterwarp_box.generic.x    =       0;
        s_gl_waterwarp_box.generic.y    =       12.5 * CHARHEIGHT;
        s_gl_waterwarp_box.generic.name =       "Water Texture Shader";
        s_gl_waterwarp_box.generic.callback =   glwaterwarpcallback;
        s_gl_waterwarp_box.generic.statusbar =  "gl_waterwarp";
        s_gl_waterwarp_box.curvalue =   gl_waterwarp->value;
        s_gl_waterwarp_box.itemnames =  yesno_names;

        
        s_gl_waterwaves_box.generic.type =      MTYPE_SPINCONTROL;
        s_gl_waterwaves_box.generic.x    =      0;
        s_gl_waterwaves_box.generic.y    =      13.75 * CHARHEIGHT;
        s_gl_waterwaves_box.generic.name =      "Water Waves";
        s_gl_waterwaves_box.generic.callback =  glwaterwavescallback;
        s_gl_waterwaves_box.generic.statusbar = "gl_waterwaves";
        s_gl_waterwaves_box.curvalue =          gl_waterwaves->value;
        s_gl_waterwaves_box.itemnames =         yesno_names;
        

        //Menu_AddItem( &s_software_menu, ( void * ) &s_ref_list[SOFTWARE_MENU] );
        Menu_AddItem( &s_software_menu, ( void * ) &s_mode_list[SOFTWARE_MENU] );
        Menu_AddItem( &s_software_menu, ( void * ) &s_brightness_slider[SOFTWARE_MENU] );

        Menu_AddItem( &s_software_menu, ( void * ) &s_fs_box[SOFTWARE_MENU] );
        Menu_AddItem( &s_software_menu, ( void * ) &s_screensize_slider[SOFTWARE_MENU] );
        //Menu_AddItem( &s_software_menu, ( void * ) &s_stipple_box );

        //Menu_AddItem( &s_opengl_menu, ( void * ) &s_ref_list[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_mode_list[OPENGL_MENU] );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_tfilter_box );

        Menu_AddItem( &s_opengl_menu, ( void * ) &s_fs_box[OPENGL_MENU] );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_brightness_slider[OPENGL_MENU] );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_screensize_slider[OPENGL_MENU] );

        //Menu_AddItem( &s_opengl_menu, ( void * ) &s_tq_slider );
        
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_finish_box );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_paletted_texture_box );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_gl_waterwarp_box );
        Menu_AddItem( &s_opengl_menu, ( void * ) &s_gl_waterwaves_box );

        //Menu_AddItem( &s_software_menu, ( void * ) &s_defaults_action[SOFTWARE_MENU] );
        Menu_AddItem( &s_software_menu, ( void * ) &s_cancel_action[SOFTWARE_MENU] );
        //Menu_AddItem( &s_opengl_menu, ( void * ) &s_defaults_action[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_cancel_action[OPENGL_MENU] );

        Menu_Center( &s_software_menu );
	Menu_Center( &s_opengl_menu );
	s_opengl_menu.x -= 8;
        s_software_menu.x -= 8;
}

/*
================
VID_MenuDraw
================
*/
void VID_MenuDraw (void)
{
	int w, h;

	if ( s_current_menu_index == 0 )
		s_current_menu = &s_software_menu;
	else
		s_current_menu = &s_opengl_menu;

	/*
	** draw the banner
	*/
        //re.DrawGetPicSize( &w, &h, "m_banner_video" );
        //re.DrawPic( viddef.width / 2 - w / 2, viddef.height /2 - 110, "m_banner_video" );

	/*
	** move cursor to a reasonable starting position
	*/
	Menu_AdjustCursor( s_current_menu, 1 );

	/*
	** draw the menu
	*/
	Menu_Draw( s_current_menu );
}

/*
================
VID_MenuKey
================
*/
const char *VID_MenuKey( int key )
{
	menuframework_s *m = s_current_menu;
        //menucommon_s *item = ( menucommon_s * ) Menu_ItemAtCursor (m);

        //static const char *sound = "misc/menu1.wav";

	switch ( key )
	{
        case K_MOUSE2:
        case K_ESCAPE:
                //ApplyChanges( 0 );
                CancelChanges( 0 );
		return NULL;
        case K_MWHEELUP:
        case K_KP_UPARROW:
	case K_UPARROW:
		m->cursor--;
		Menu_AdjustCursor( m, -1 );
		break;
        case K_MWHEELDOWN:
        case K_KP_DOWNARROW:
	case K_DOWNARROW:
		m->cursor++;
		Menu_AdjustCursor( m, 1 );
		break;
	case K_KP_LEFTARROW:
	case K_LEFTARROW:
		Menu_SlideItem( m, -1 );
		break;
	case K_KP_RIGHTARROW:
	case K_RIGHTARROW:
		Menu_SlideItem( m, 1 );
		break;
                
        case K_MOUSE1:
        case K_KP_ENTER:
	case K_ENTER:
                //if ( !Menu_SelectItem( m ) )
                //        ApplyChanges( NULL );
                /*
                if (item)
                {
                        if (item->type == MTYPE_ACTION)
                                ApplyChanges( NULL );
                }
                */
                Menu_SelectItem( m );
                //bc
		break;
                
	}

        //return sound;
        return NULL;
        
}


